# encoding: utf-8

from yade import pack, utils, ymport, export
import numpy as np

young = 65e9
poisson = 0.3
density = 5000
mn,mx=Vector3(-0.02, -.02,-0.02),Vector3(0.02, 0.02, 0.02)
finalFricDegree = 19 # contact friction during the deviatoric loading
jointFrict=radians(18)
jointDil=radians(0)
intRadius= 1.329
sigmaT = 6e6
cohesion = 40e6
wellRad = 0.008
Shmin = -1e6  # x
SHmax = -1e6  # y 
Sv = -1e6  # z


packing = 'liteSpecimen'
DFN = 'jointSurf'

jCFmat = O.materials.append(JCFpmMat(young=young, cohesion=cohesion, density=density, frictionAngle=radians(finalFricDegree), tensileStrength=sigmaT, poisson=poisson, label='JCFmat', jointNormalStiffness=50e9,jointShearStiffness=12.5e9,jointCohesion=1e5, jointTensileStrength=1e5,jointFrictionAngle=jointFrict, jointDilationAngle=jointDil)) 

facetMat = O.materials.append(JCFpmMat(type=0,young=1,frictionAngle=radians(1),poisson=0.4,density=1))

wallmat = O.materials.append(FrictMat(young=80e9,poisson=.45,frictionAngle=radians(finalFricDegree),density=2600,label='frictionlessWalls'))

walls=aabbWalls([mn,mx], material='frictionlessWalls')
wallIds=O.bodies.append(walls)

sp = O.bodies.append(ymport.textExt('liteSpecimen2mm.spheres', 'x_y_z_r',color=(0.9,0.1,0.6), material='JCFmat'))

frac = O.bodies.append(ymport.stl(DFN+'.stl',color=(0.9,0.9,0.9),wire=False,material=facetMat)) 


triax = TriaxialStressController(	
	thickness=0,
	stressMask=7,
	internalCompaction=False,

	)

O.engines=[
	ForceResetter(),
	InsertionSortCollider([Bo1_Sphere_Aabb(aabbEnlargeFactor=intRadius),Bo1_Box_Aabb(), Bo1_Facet_Aabb()]),
	InteractionLoop(
		[Ig2_Sphere_Sphere_ScGeom6D(interactionDetectionFactor=intRadius),Ig2_Box_Sphere_ScGeom(),Ig2_Facet_Sphere_ScGeom()],
		[Ip2_FrictMat_FrictMat_FrictPhys(),Ip2_JCFpmMat_JCFpmMat_JCFpmPhys(cohesiveTresholdIteration=-1,label='jcf')],
		[Law2_ScGeom_JCFpmPhys_JointedCohesiveFrictionalPM(Key="hydrofrac", smoothJoint=True,label='interactionLaw', recordCracks=True),Law2_ScGeom_FrictPhys_CundallStrack()]
	),
	DFNFlowEngine(multithread=True,dead=1,label="flow", wallIds=wallIds),	
	GlobalStiffnessTimeStepper(active=1,timeStepUpdateInterval=100,timestepSafetyCoefficient=0.5),
	triax,
	NewtonIntegrator(damping=.4)
]


O.dt = 0.5 * utils.PWaveTimeStep()

triax.goal1=SHmax
triax.goal2=Sv
triax.goal3=Shmin
O.step()


jointcolor1=(0,1,0)
jointcolor2=(1,0,0)
jointcolor3=(0,0,1)
jointcolor4=(1,1,1)
jointcolor5=(0,0,0)
onJfnd = 0;

#### first step-> find spheres on facet
for i in O.interactions:
    ##if not i.isReal : continue
    if isinstance(O.bodies[i.id1].shape,Facet) and isinstance(O.bodies[i.id2].shape,Sphere): 
	vertices=O.bodies[i.id1].shape.vertices
	normalRef=vertices[0].cross(vertices[1]) # defines the normal to the facet normalRef
	nRef=normalRef/(normalRef.norm()) ## normalizes normalRef
	normalFacetSphere=i.geom.normal # geom.normal is oriented from id1 to id2 -> normalFacetSphere from facet (i.id1) to sphere (i.id2)

	if O.bodies[i.id2].state.onJoint==False : ## particles has not yet been identified as belonging to a joint plane
	    O.bodies[i.id2].state.onJoint=True
	    onJfnd += 1
	    O.bodies[i.id2].state.joint=1
	    O.bodies[i.id2].shape.color=jointcolor1
	    if nRef.dot(normalFacetSphere)>=0 :
		O.bodies[i.id2].state.jointNormal1=nRef
	    elif nRef.dot(normalFacetSphere)<0 :
		O.bodies[i.id2].state.jointNormal1=-nRef
	elif O.bodies[i.id2].state.onJoint==True :  ## particles has already been identified as belonging to, at least, 1 facet
	    if O.bodies[i.id2].state.joint==1 and ((O.bodies[i.id2].state.jointNormal1.cross(nRef)).norm()>0.05) : ## particles has already been identified as belonging to only 1 facet
		O.bodies[i.id2].state.joint=2
		O.bodies[i.id2].shape.color=jointcolor2
		if nRef.dot(normalFacetSphere)>=0 :
		    O.bodies[i.id2].state.jointNormal2=nRef
		elif nRef.dot(normalFacetSphere)<0 :
		    O.bodies[i.id2].state.jointNormal2=-nRef
	    elif O.bodies[i.id2].state.joint==2 and ((O.bodies[i.id2].state.jointNormal1.cross(nRef)).norm()>0.05) and ((O.bodies[i.id2].state.jointNormal2.cross(nRef)).norm()>0.05): ## particles has already been identified as belonging to more than 1 facet
		O.bodies[i.id2].state.joint=3
		O.bodies[i.id2].shape.color=jointcolor3
		if nRef.dot(normalFacetSphere)>=0 :
		    O.bodies[i.id2].state.jointNormal3=nRef
		elif nRef.dot(normalFacetSphere)<0 :
		    O.bodies[i.id2].state.jointNormal3=-nRef
	    elif O.bodies[i.id2].state.joint==3 and ((O.bodies[i.id2].state.jointNormal1.cross(nRef)).norm()>0.05) and ((O.bodies[i.id2].state.jointNormal2.cross(nRef)).norm()>0.05) and ((O.bodies[i.id2].state.jointNormal3.cross(nRef)).norm()>0.05):
		O.bodies[i.id2].state.joint=4
		O.bodies[i.id2].shape.color=jointcolor5

#### second step -> find spheres interacting with spheres on facet (could be executed in the same timestep as step 1?)
for j in O.interactions:
    #if not i.isReal : continue
    if isinstance(O.bodies[j.id1].shape,Facet) and isinstance(O.bodies[j.id2].shape,Sphere):
	vertices=O.bodies[j.id1].shape.vertices
	normalRef=vertices[0].cross(vertices[1]) # defines the normal to the facet normalRef
	nRef=normalRef/(normalRef.norm()) ## normalizes normalRef
	if ((O.bodies[j.id2].state.jointNormal1.cross(nRef)).norm()<0.05) :
	    jointNormalRef=O.bodies[j.id2].state.jointNormal1
	elif ((O.bodies[j.id2].state.jointNormal2.cross(nRef)).norm()<0.05) :
	    jointNormalRef=O.bodies[j.id2].state.jointNormal2
	elif ((O.bodies[j.id2].state.jointNormal3.cross(nRef)).norm()<0.05) :
	    jointNormalRef=O.bodies[j.id2].state.jointNormal3
	else : continue
	facetCenter=O.bodies[j.id1].state.pos
	#### seek for each sphere interacting with the identified sphere j.id2
	for n in O.interactions.withBody(j.id2) :
	    if n.id1==j.id2 and isinstance(O.bodies[n.id2].shape,Sphere): 
		facetSphereDir=(O.bodies[n.id2].state.pos-facetCenter)
		if O.bodies[n.id2].state.onJoint==True :
		    if O.bodies[n.id2].state.joint==3 and ((O.bodies[n.id2].state.jointNormal1.cross(jointNormalRef)).norm()>0.05) and ((O.bodies[n.id2].state.jointNormal2.cross(jointNormalRef)).norm()>0.05) and ((O.bodies[n.id2].state.jointNormal3.cross(jointNormalRef)).norm()>0.05):
			O.bodies[n.id2].state.joint=4
			O.bodies[n.id2].shape.color=jointcolor5
		    elif O.bodies[n.id2].state.joint==2 and ((O.bodies[n.id2].state.jointNormal1.cross(jointNormalRef)).norm()>0.05) and ((O.bodies[n.id2].state.jointNormal2.cross(jointNormalRef)).norm()>0.05):
			O.bodies[n.id2].state.joint=3
			if facetSphereDir.dot(jointNormalRef)>=0:
			    O.bodies[n.id2].state.jointNormal3=jointNormalRef
			elif facetSphereDir.dot(jointNormalRef)<0:
			    O.bodies[n.id2].state.jointNormal3=-jointNormalRef 
		    elif O.bodies[n.id2].state.joint==1 and ((O.bodies[n.id2].state.jointNormal1.cross(jointNormalRef)).norm()>0.05) :
			O.bodies[n.id2].state.joint=2
			if facetSphereDir.dot(jointNormalRef)>=0:
			    O.bodies[n.id2].state.jointNormal2=jointNormalRef
			elif facetSphereDir.dot(jointNormalRef)<0:
			    O.bodies[n.id2].state.jointNormal2=-jointNormalRef
		elif  O.bodies[n.id2].state.onJoint==False :
		    O.bodies[n.id2].state.onJoint=True
		    O.bodies[n.id2].state.joint=1
		    O.bodies[n.id2].shape.color=jointcolor4
		    if facetSphereDir.dot(jointNormalRef)>=0:
			O.bodies[n.id2].state.jointNormal1=jointNormalRef
		    elif facetSphereDir.dot(jointNormalRef)<0:
			O.bodies[n.id2].state.jointNormal1=-jointNormalRef

	    elif n.id2==j.id2 and isinstance(O.bodies[n.id1].shape,Sphere): 
		facetSphereDir=(O.bodies[n.id1].state.pos-facetCenter)
		if O.bodies[n.id1].state.onJoint==True :
		    if O.bodies[n.id1].state.joint==3 and ((O.bodies[n.id1].state.jointNormal1.cross(jointNormalRef)).norm()>0.05) and ((O.bodies[n.id1].state.jointNormal2.cross(jointNormalRef)).norm()>0.05) and ((O.bodies[n.id1].state.jointNormal3.cross(jointNormalRef)).norm()>0.05):
			O.bodies[n.id1].state.joint=4
			O.bodies[n.id1].shape.color=jointcolor5
		    elif O.bodies[n.id1].state.joint==2 and ((O.bodies[n.id1].state.jointNormal1.cross(jointNormalRef)).norm()>0.05) and ((O.bodies[n.id1].state.jointNormal2.cross(jointNormalRef)).norm()>0.05):
			O.bodies[n.id1].state.joint=3
			if facetSphereDir.dot(jointNormalRef)>=0:
			    O.bodies[n.id1].state.jointNormal3=jointNormalRef
			elif facetSphereDir.dot(jointNormalRef)<0:
			    O.bodies[n.id1].state.jointNormal3=-jointNormalRef
		    elif O.bodies[n.id1].state.joint==1 and ((O.bodies[n.id1].state.jointNormal1.cross(jointNormalRef)).norm()>0.05) :
			O.bodies[n.id1].state.joint=2
			if facetSphereDir.dot(jointNormalRef)>=0:
			    O.bodies[n.id1].state.jointNormal2=jointNormalRef
			elif facetSphereDir.dot(jointNormalRef)<0:
			    O.bodies[n.id1].state.jointNormal2=-jointNormalRef
		elif  O.bodies[n.id1].state.onJoint==False :
		    O.bodies[n.id1].state.onJoint=True
		    O.bodies[n.id1].state.joint=1
		    O.bodies[n.id1].shape.color=jointcolor4
		    if facetSphereDir.dot(jointNormalRef)>=0:
			O.bodies[n.id1].state.jointNormal1=jointNormalRef
		    elif facetSphereDir.dot(jointNormalRef)<0:
			O.bodies[n.id1].state.jointNormal1=-jointNormalRef
##### to delete facets
for b in O.bodies:
    if isinstance(b.shape,Facet):
	O.bodies.erase(b.id)


O.interactions.clear()

O.step()

flow.debug= True
flow.dead=0
flow.permeabilityMap = True
flow.pZero = 0.  # 5e6
#flow.defTolerance=0.3
#flow.meshUpdateInterval=50
flow.useSolver=3
flow.permeabilityFactor=5e-4
flow.viscosity=1
flow.fluidBulkModulus = 2.2e9
flow.bndCondIsPressure=[1,1,1,1,1,1]
flow.bndCondValue=[0,0,0,0,0,0]
##flow.boundaryUseMaxMin=[0,0,0,0,0,0]

flow.viscousNormalBodyStress=True
flow.viscousShearBodyStress=True
flow.jointResidual=10e-4 # force aperture permeability to be 1e-6 for residual openings # 10**-6


O.run(10)



